%% PROCESS LUMINANCE MEASUREMENTS
clc;
fprintf('Processing luminance measurements for the BOLD screen\n\n');

% load the measurements
load('luminance-curve_2021S10_1524.mat');tbl1=tbl;
tbl1.Properties.VariableNames = {'in_n' 'in_dac' 'out_lum' 'out_n'};
mL1=maxLevel;dB1=displayBaseline;dR1=displayRange;
load('luminance-curve_2021S10_1530.mat');tbl2=tbl;
tbl2.Properties.VariableNames = {'in_n' 'in_dac' 'out_lum' 'out_n'};
mL2=maxLevel;dB2=displayBaseline;dR2=displayRange;
load('luminance-curve_2021S10_1534.mat');tbl3=tbl;
tbl3.Properties.VariableNames = {'in_n' 'in_dac' 'out_lum' 'out_n'};
mL3=maxLevel;dB3=displayBaseline;dR3=displayRange;

tbl=[tbl1;tbl2;tbl3];
mL=mean([mL1 mL2 mL3]);
dB=mean([dB1 dB2 dB3]);
dR=mean([dR1 dR2 dR3]);

%% plot

% dac to luminance
g = fittype([num2str(dB) '+ ( (x/' num2str(mL) ')^b).*' num2str(dR)]); 
warning off;

fm1 = fit(tbl1.in_dac,tbl1.out_lum,g);
fm2 = fit(tbl2.in_dac,tbl2.out_lum,g);
fm3 = fit(tbl3.in_dac,tbl3.out_lum,g);
fm = fit(tbl.in_dac,tbl.out_lum,g);

f1=figure;
set(f1,'Position',[100 100 1000 900]);
subplot(2,2,1);hold on;
plot(tbl.in_dac,tbl.out_lum,'ko','MarkerSize',10);
plot(0:0.1:255,fm(0:0.1:255),'r-','LineWidth',3');
set(gca,'xlim',[0 255]);
xlabel('DAC value in'); 
ylabel('Luminance (cd/m2)');
title('bit to luminance')
legend({'Measured','Fit'},'Location','NorthWest','Interpreter','none');

% normalized i/o
gn = fittype('x^b'); warning off;
fmn1 = fit(tbl1.in_n,tbl1.out_n,gn);
fmn2 = fit(tbl2.in_n,tbl2.out_n,gn);
fmn3 = fit(tbl3.in_n,tbl3.out_n,gn);
fmn = fit(tbl.in_n,tbl.out_n,gn);

subplot(2,2,2);hold on;
plot(tbl.in_n,tbl.out_n,'ko','MarkerSize',10);
plot(0:0.01:1,fmn(0:0.01:1),'b-','LineWidth',3');
set(gca,'xlim',[0 1]);
title('normalized i/o'); xlabel('Norm in'); ylabel('Norm out');
legend({'Measured',['Fit f(x)=x^' ...
    num2str(fmn.b)]},'Location','NorthWest','Interpreter','none');

% inverse --
%y = x.^b
%b = logx(y)
%b = log(y)/log(x)
%log(x) = log(y)/b
%x = exp(log(y)/b)
%x = y/exp(b)

yn = 0:0.1:1;
xx = exp(log(yn)/fmn.b);
yy = fmn(xx);

% norm o/i
f2=figure; hold on;
plot(tbl.out_n,tbl.in_n,'ko','MarkerSize',10);
plot(0:0.01:1,exp(log(0:0.01:1)/fmn.b),'g-','LineWidth',3');
set(gca,'xlim',[0 1]);
title('normalized o/i'); 
xlabel('norm lum'); 
ylabel('norm DAC');
close(f2)

figure(f1)

% linearize
linx = @(x,b)exp(log(x)/b);

% linearized norm i/o
subplot(2,2,3);hold on;
plot(tbl.in_n,fmn(linx(tbl.in_n,fmn.b)),'ko','MarkerSize',10);
plot(0:0.01:1,fmn(linx(0:0.01:1,fmn.b)),'b-','LineWidth',3');
set(gca,'xlim',[0 1]);
title('linearized i/o'); 
xlabel('linearizednorm DAC'); 
ylabel('Norm lum');

linstring1 = 'Linear DAC =';
linstring2 = ['exp(log(Uncorr DAC)/' num2str(fmn.b) ')'];
text(0.05,0.95,linstring1);
text(0.05,0.90,linstring2);


% linearized absolute i/o
alum = @(nlum,dB,dR)dB + nlum*dR;
adac = @(ndac,mL)ndac.*mL;

subplot(2,2,4);hold on;
plot(adac(tbl.in_n,mL),...
    alum(fmn(linx(tbl.in_n,fmn.b)),dB,dR),...
    'ko','MarkerSize',10);
plot(adac(0:0.01:1,mL),...
    alum(fmn(linx(0:0.01:1,fmn.b)),dB,dR),...
    'r-','LineWidth',3');
set(gca,'xlim',[0 255]);
title('linearized i/o'); 
xlabel('Linearized DAC value'); 
ylabel('Luminance (cd/m2)');

saveas(f1,'LuminanceCurves.fig');
saveas(f1,'LuminanceCurves.png');

%% Print some conclusions to text file and show output
fid = fopen('LUMINANCE-CURVE_Summary.txt','w');
fprintf(fid,'==================================================================\n');
fprintf(fid,['SUMMARY OF LUMINANCE CURVES AND LINEARIZATION\n'...
    '-- BOLD screen Spinoza Center --\n']);
fprintf(fid,'==================================================================\n');

fprintf(fid,'Gamma curve of the uncorrected BOLD display\n\n');

fprintf(fid,'-- NORMALIZED --\n');
fprintf(fid,['NORMALIZED LUMINANCE = NORMALIZED DAC-VALUE ^ ' num2str(fmn.b) '\n\n' ]);

fprintf(fid,'-- ABSOLUTE --\n');
fprintf(fid,['LUMINANCE (cd/m2) = ' num2str(dB) ' + ' num2str(dR) ...
    '*((DAC-VALUE/' num2str(mL) ')^' num2str(fmn.b) ')\n\n' ]);

fprintf(fid,'Create a normalized gamma table with:\n');
fprintf(fid,['LINEARIZED DAC-VALUE = EXP( LOG(UNCORRECTED DAC-VALUE)/' num2str(fmn.b) ')\n\n']);

fprintf(fid,'==================================================================\n');

fclose(fid);
summary=fileread('LUMINANCE-CURVE_Summary.txt')